
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '36.0.0.post2',
    'branch_nick': 'release-36',
    'revision_id': 'b07377d21b4df2a3ffd4908a32c504b453de3fa8',
    'revno': 13375
}
